@@ -146,7 +146,7 @@ impl CertSynopsis {
     /// Only valid, self-signed User IDs are returned.
     ///
     /// The primary User ID is returned first.
-    pub fn userids<'a>(&'a self) -> impl Iterator<Item=&UserIDSynopsis> + 'a {
+    pub fn userids(&self) -> impl Iterator<Item=&UserIDSynopsis> {
         self.userids.iter()
     }
 
