@@ -147,6 +147,15 @@ impl Depth {
         matches!(self, Depth::Unconstrained)
     }
 
+    /// Returns whether this `Depth` allows introducing.
+    pub fn can_introduce(&self) -> bool {
+        match self {
+            Depth::Unconstrained => true,
+            Depth::Limit(d) if *d > 0 => true,
+            _ => false
+        }
+    }
+
     /// Converts the `Depth` to an `Option<usize>`.
     ///
     /// An unconstrained depth is converted to `None`.  A constrained
