@@ -192,13 +192,13 @@ impl<S> Network<S>
     }
 }
 
-impl<'a: 'ra, 'ra> Network<CertStore<'a, 'ra, cert_store::store::Certs<'a>>> {
+impl<'a: 'policy, 'policy> Network<CertStore<'a, 'policy, cert_store::store::Certs<'a>>> {
     /// Builds a web of trust network from a set of certificates.
     ///
     /// If a certificate is invalid according to the policy, the
     /// certificate is silently ignored.
     pub fn from_certs<I, C, T, R>(certs: I,
-                                  policy: &'ra dyn Policy, t: T,
+                                  policy: &'policy dyn Policy, t: T,
                                   roots: R)
         -> Result<Self>
     where T: Into<Option<SystemTime>>,
@@ -221,7 +221,7 @@ impl<'a: 'ra, 'ra> Network<CertStore<'a, 'ra, cert_store::store::Certs<'a>>> {
     /// If a certificate is invalid according to the policy, the
     /// certificate is silently ignored.
     pub fn from_cert_refs<I, C, T, R>(certs: I,
-                                      policy: &'ra dyn Policy, t: T,
+                                      policy: &'policy dyn Policy, t: T,
                                       roots: R)
         -> Result<Self>
     where T: Into<Option<SystemTime>>,
@@ -243,7 +243,7 @@ impl<'a: 'ra, 'ra> Network<CertStore<'a, 'ra, cert_store::store::Certs<'a>>> {
     ///
     /// If a certificate is invalid according to the policy, the
     /// certificate is silently ignored.
-    pub fn from_bytes<T, R>(certs: &'a [u8], policy: &'ra dyn Policy, t: T,
+    pub fn from_bytes<T, R>(certs: &'a [u8], policy: &'policy dyn Policy, t: T,
                             roots: R)
         -> Result<Self>
     where T: Into<Option<SystemTime>>,
