@@ -30,7 +30,7 @@ use crate::store::Store;
 
 use super::TRACE;
 
-const NP: NullPolicy = NullPolicy::new();
+const NP: NullPolicy = unsafe { NullPolicy::new() };
 
 /// [`Network::lint_path`] specific error codes.
 #[non_exhaustive]
