@@ -152,8 +152,8 @@ impl Path {
     /// Returns an iterator over the path's certificates (the nodes).
     ///
     /// The certificates are returned from the root towards the target.
-    pub fn certificates<'a>(&'a self)
-        -> impl 'a + Iterator<Item=&CertSynopsis> + DoubleEndedIterator
+    pub fn certificates(&self)
+        -> impl Iterator<Item=&CertSynopsis> + DoubleEndedIterator
     {
         std::iter::once(&self.root)
             .chain(self.edges.iter().map(|certification| {
@@ -169,13 +169,18 @@ impl Path {
     /// Returns the certifications.
     ///
     /// The certifications are returned from the root towards the target.
-    pub fn certifications<'a>(&'a self) -> impl Iterator<Item=&Certification> {
+    pub fn certifications(&self) -> impl Iterator<Item=&Certification> {
         self.edges.iter()
     }
 
     /// Returns the residual trust depth.
-    #[cfg(test)]
-    pub(crate) fn residual_depth(&self) -> Depth {
+    ///
+    /// The residual trust depth is calculated by taking the trust
+    /// depth of root and reducing it by one for each certification in
+    /// the path.  If passing a certification with a lower trust depth
+    /// than the current trust value, the lower value is used from
+    /// that point on.
+    pub fn residual_depth(&self) -> Depth {
         self.residual_depth
     }
 
