@@ -28,7 +28,7 @@ use crate::Depth;
 use crate::TRACE;
 
 mod cert_store;
-pub use cert_store::CertStore;
+pub use cert_store::{CertStore, MaybeOwnedPolicy};
 mod synopses;
 pub use synopses::SynopsisSlice;
 
@@ -601,7 +601,7 @@ pub trait Store {
     /// shouldn't be associated with that User ID.
     fn lookup_synopses_by_email(&self, email: &str) -> Vec<(Fingerprint, UserID)> {
         let userid_check = UserID::from(format!("<{}>", email));
-        if let Ok(Some(email_check)) = userid_check.email2() {
+        if let Ok(Some(email_check)) = userid_check.email() {
             if email != email_check {
                 // Does not appear to be an email address.
                 return Vec::new();
