@@ -120,7 +120,7 @@ use crate as wot;
 /// // And thanks to nll, we can immediately transfer ownership.
 /// drop(cert_store);
 /// # Ok(()) }
-pub struct CertStore<'a: 'ra, 'ra, S>
+pub struct CertStore<'a: 'policy, 'policy, S>
     where S: cert_store::store::Store<'a>,
 {
     store: S,
@@ -144,7 +144,7 @@ pub struct CertStore<'a: 'ra, 'ra, S>
 
     // The policy.  This is needed to compute the
     // certification sets.
-    policy: &'ra dyn Policy,
+    policy: MaybeOwnedPolicy<'policy>,
 
     // The reference time.  This is needed to compute the
     // certification sets.
@@ -153,8 +153,55 @@ pub struct CertStore<'a: 'ra, 'ra, S>
     _a: std::marker::PhantomData<&'a ()>,
 }
 
+/// The [`CertStore`] may either borrow or own a policy.
+#[non_exhaustive]
+pub enum MaybeOwnedPolicy<'p> {
+    /// A borrowed policy.
+    Borrowed(&'p dyn Policy),
 
-impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
+    /// A shared policy.
+    Arc(Arc<dyn Policy + 'p>),
+
+    /// An owned policy.
+    Owned(Box<dyn Policy + 'p>),
+}
+
+impl MaybeOwnedPolicy<'_> {
+    /// Returns the policy.
+    fn as_ref(&self) -> &dyn Policy {
+        match self {
+            MaybeOwnedPolicy::Borrowed(p) => *p,
+            MaybeOwnedPolicy::Arc(p) => p.as_ref(),
+            MaybeOwnedPolicy::Owned(p) => p.as_ref(),
+        }
+    }
+}
+
+impl<'p> From<&'p dyn Policy> for MaybeOwnedPolicy<'p> {
+    fn from(p: &'p dyn Policy) -> Self {
+        MaybeOwnedPolicy::Borrowed(p)
+    }
+}
+
+impl<'p, P: Policy> From<&'p P> for MaybeOwnedPolicy<'p> {
+    fn from(p: &'p P) -> Self {
+        MaybeOwnedPolicy::Borrowed(&*p)
+    }
+}
+
+impl<'p> From<Arc<dyn Policy + 'p>> for MaybeOwnedPolicy<'p> {
+    fn from(p: Arc<dyn Policy + 'p>) -> Self {
+        MaybeOwnedPolicy::Arc(p)
+    }
+}
+
+impl<'p> From<Box<dyn Policy + 'p>> for MaybeOwnedPolicy<'p> {
+    fn from(p: Box<dyn Policy + 'p>) -> Self {
+        MaybeOwnedPolicy::Owned(p)
+    }
+}
+
+impl<'a: 'policy, 'policy, S> CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a>,
 {
     /// Returns a new `CertStore` from a cert store.
@@ -164,16 +211,18 @@ impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
     ///
     /// [`store::Backend`]: crate::store::Backend
     /// [`store::Store`]: crate::store::Store
-    pub fn from_store<T>(store: S, policy: &'ra dyn Policy, t: T)
+    pub fn from_store<P, T>(store: S, policy: P, t: T)
         -> Self
-    where T: Into<Option<SystemTime>>,
+    where
+        P: Into<MaybeOwnedPolicy<'policy>>,
+        T: Into<Option<SystemTime>>,
     {
         let t = t.into().unwrap_or_else(SystemTime::now);
 
         Self {
             store,
             redge_cache: Default::default(),
-            policy,
+            policy: policy.into(),
             reference_time: t,
             _a: std::marker::PhantomData,
         }
@@ -195,8 +244,8 @@ impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
     }
 
     /// Returns the configured policy.
-    pub fn policy(&self) -> &'ra dyn Policy {
-        self.policy
+    pub fn policy(&self) -> &dyn Policy {
+        self.policy.as_ref()
     }
 
     /// Returns the configured reference time.
@@ -205,7 +254,7 @@ impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
     }
 }
 
-impl<'a: 'ra, 'ra, S> std::ops::Deref for CertStore<'a, 'ra, S>
+impl<'a: 'policy, 'policy, S> std::ops::Deref for CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a>,
 {
     type Target = S;
@@ -215,7 +264,7 @@ impl<'a: 'ra, 'ra, S> std::ops::Deref for CertStore<'a, 'ra, S>
     }
 }
 
-impl<'a: 'ra, 'ra, S> std::ops::DerefMut for CertStore<'a, 'ra, S>
+impl<'a: 'policy, 'policy, S> std::ops::DerefMut for CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a>,
 {
     fn deref_mut(&mut self) -> &mut Self::Target {
@@ -223,16 +272,17 @@ impl<'a: 'ra, 'ra, S> std::ops::DerefMut for CertStore<'a, 'ra, S>
     }
 }
 
-impl<'a: 'ra, 'ra> CertStore<'a, 'ra, cert_store::store::Certs<'a>>
+impl<'a: 'policy, 'policy> CertStore<'a, 'policy, cert_store::store::Certs<'a>>
 {
     /// Returns a new `CertStore` from a slice of bytes.
     ///
     /// The bytes are interpreted as an OpenPGP keyring.  The data
     /// may, but need not, be ASCII-armor encoded.
-    pub fn from_bytes<T>(bytes: &'a [u8],
-                         policy: &'ra dyn Policy, t: T)
+    pub fn from_bytes<P, T>(bytes: &'a [u8], policy: P, t: T)
         -> Result<Self>
-    where T: Into<Option<SystemTime>>,
+    where
+        P: Into<MaybeOwnedPolicy<'policy>>,
+        T: Into<Option<SystemTime>>,
     {
         tracer!(TRACE, "CertStore::from_bytes");
 
@@ -243,17 +293,19 @@ impl<'a: 'ra, 'ra> CertStore<'a, 'ra, cert_store::store::Certs<'a>>
         Ok(Self {
             store,
             redge_cache: Default::default(),
-            policy,
+            policy: policy.into(),
             reference_time: t,
             _a: std::marker::PhantomData,
         })
     }
 
     /// Creates a `CertStore` from some `Cert`s.
-    pub fn from_certs<T>(certs: impl IntoIterator<Item=Cert>,
-                         policy: &'ra dyn Policy, t: T)
+    pub fn from_certs<P, T>(certs: impl IntoIterator<Item=Cert>,
+                            policy: P, t: T)
         -> Result<Self>
-    where T: Into<Option<SystemTime>>,
+    where
+        P: Into<MaybeOwnedPolicy<'policy>>,
+        T: Into<Option<SystemTime>>,
     {
         tracer!(TRACE, "CertStore::from_certs");
 
@@ -264,17 +316,19 @@ impl<'a: 'ra, 'ra> CertStore<'a, 'ra, cert_store::store::Certs<'a>>
         Ok(Self {
             store,
             redge_cache: Default::default(),
-            policy,
+            policy: policy.into(),
             reference_time: t,
             _a: std::marker::PhantomData,
         })
     }
 
     /// Creates a `CertStore` from `&Cert`s.
-    pub fn from_cert_refs<T>(certs: impl IntoIterator<Item=&'a Cert>,
-                             policy: &'ra dyn Policy, t: T)
+    pub fn from_cert_refs<P, T>(certs: impl IntoIterator<Item=&'a Cert>,
+                                policy: P, t: T)
         -> Result<Self>
-    where T: Into<Option<SystemTime>>,
+    where
+        P: Into<MaybeOwnedPolicy<'policy>>,
+        T: Into<Option<SystemTime>>,
     {
         tracer!(TRACE, "CertStore::from_cert_refs");
 
@@ -289,17 +343,19 @@ impl<'a: 'ra, 'ra> CertStore<'a, 'ra, cert_store::store::Certs<'a>>
         Ok(Self {
             store,
             redge_cache: Default::default(),
-            policy,
+            policy: policy.into(),
             reference_time: t,
             _a: std::marker::PhantomData,
         })
     }
 
     /// Creates a `CertStore` from `RawCert`s.
-    pub fn from_raw_certs<T>(certs: impl IntoIterator<Item=RawCert<'a>>,
-                             policy: &'ra dyn Policy, t: T)
+    pub fn from_raw_certs<P, T>(certs: impl IntoIterator<Item=RawCert<'a>>,
+                                policy: P, t: T)
         -> Result<Self>
-    where T: Into<Option<SystemTime>>
+    where
+        P: Into<MaybeOwnedPolicy<'policy>>,
+        T: Into<Option<SystemTime>>,
     {
         tracer!(TRACE, "CertStore::from_raw_certs");
 
@@ -310,14 +366,14 @@ impl<'a: 'ra, 'ra> CertStore<'a, 'ra, cert_store::store::Certs<'a>>
         Ok(Self {
             store,
             redge_cache: Default::default(),
-            policy,
+            policy: policy.into(),
             reference_time: t,
             _a: std::marker::PhantomData,
         })
     }
 }
 
-impl<'a: 'ra, 'ra, S> cert_store::Store<'a> for CertStore<'a, 'ra, S>
+impl<'a: 'policy, 'policy, S> cert_store::Store<'a> for CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a>,
 {
     fn lookup_by_cert(&self, kh: &KeyHandle) -> Result<Vec<Arc<LazyCert<'a>>>> {
@@ -380,7 +436,7 @@ impl<'a: 'ra, 'ra, S> cert_store::Store<'a> for CertStore<'a, 'ra, S>
     }
 }
 
-impl<'a: 'ra, 'ra, S> cert_store::StoreUpdate<'a> for CertStore<'a, 'ra, S>
+impl<'a: 'policy, 'policy, S> cert_store::StoreUpdate<'a> for CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a> + cert_store::store::StoreUpdate<'a>,
 {
     fn update_by(&self,
@@ -446,7 +502,7 @@ impl<'a: 'ra, 'ra, S> cert_store::StoreUpdate<'a> for CertStore<'a, 'ra, S>
     }
 }
 
-impl<'a: 'ra, 'ra, S> Backend<'a> for CertStore<'a, 'ra, S>
+impl<'a: 'policy, 'policy, S> Backend<'a> for CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a> + Send + Sync,
 {
     /// Prefills the cache.
@@ -510,7 +566,7 @@ impl<'a: 'ra, 'ra, S> Backend<'a> for CertStore<'a, 'ra, S>
                 // Reborrow to get a normal reference, which can be
                 // copied to the threads.
                 let backend = &*self;
-                let policy = self.policy;
+                let policy = self.policy();
                 let reference_time = self.reference_time.clone();
 
                 threads_extant.push(thread_scope.spawn(move |_| {
@@ -584,7 +640,7 @@ impl<'a: 'ra, 'ra, S> Backend<'a> for CertStore<'a, 'ra, S>
     }
 }
 
-impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
+impl<'a: 'policy, 'policy, S> CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a> + Send + Sync,
 {
     fn certifications_of_uncached<F>(&self, target: F)
@@ -594,7 +650,7 @@ impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
         let target = target.borrow();
         let cert = self.store.lookup_by_cert_fpr(target)?;
         let redges = self.redges(
-            cert.with_policy(self.policy, self.reference_time)?,
+            cert.with_policy(self.policy(), self.reference_time)?,
             0.into());
 
         Ok(redges)
@@ -604,14 +660,14 @@ impl<'a: 'ra, 'ra, S> CertStore<'a, 'ra, S>
         cert
             .to_cert()
             .and_then(|c| {
-                c.with_policy(self.policy, self.reference_time)
+                c.with_policy(self.policy(), self.reference_time)
             })
             .map(Into::into)
             .ok()
     }
 }
 
-impl<'a: 'ra, 'ra, S> Store for CertStore<'a, 'ra, S>
+impl<'a: 'policy, 'policy, S> Store for CertStore<'a, 'policy, S>
     where S: cert_store::store::Store<'a> + Send + Sync,
 {
     fn reference_time(&self) -> SystemTime {
@@ -781,10 +837,10 @@ mod test {
     // Check that that lifetimes allow us to use a CertStore as follow:
     #[test]
     fn cert_store_lifetimes() -> Result<()> {
-        fn authenticate<'store: 'ra, 'ra>(
+        fn authenticate<'store: 'policy, 'policy>(
             // The core of our check.  Does the following compile:
-            store: &CertStore<'store, 'ra,
-                              &'ra cert_store::CertStore<'store>>,
+            store: &CertStore<'store, 'policy,
+                              &'policy cert_store::CertStore<'store>>,
             trust_root: Fingerprint,
             target_fpr: Fingerprint,
             target_userid: UserID)
